import { useState, useEffect } from 'react';
import { useNavigate } from 'react-router-dom';
import { User, Accounts, Cards, Transfers, Receivers } from '../types';
import axios from 'axios';

import TopBar from '../components/TopBar';

export default function Home () {
    const navigate = useNavigate();
    const [user, setUser] = useState<User>();
    const [transfers, setTransfers] = useState<Transfers>([]);
    const [accounts, setAccounts] = useState<Accounts>([]);
    const [cards, setCards] = useState<Cards>([]);
    const [receivers, setReceivers] = useState<Receivers>([]);
    const [accountId, setAccountId] = useState<number>(0);
    const [accountToId, setAccountToId] = useState<number>(0);
    const [amount, setAmount] = useState<number>(0);
    const [message, setMessage] = useState<string>("");
    const [receiverId, setReceiverId] = useState<number>(0);
    const [receiverName, setReceiverName] = useState<string>("");
    const token = localStorage.getItem("token");

    function sendMoney (e: React.FormEvent) {
        e.preventDefault();
        
        axios
            .post("/api/@me/send-money", {
                token: token,
                account_from_id: accountId,
                account_to_id: accountToId,
                amount: amount,
                name: message
            })
            .then(() => {
                setAmount(0);
                setMessage("");
                axios
                    .post("/api/@me/accounts", { token: token })
                    .then((response) => {
                        setAccounts(response.data);
                        if (response.data.length > 0) {
                            setAccountId(response.data[0].id);
                        }
                    })
                    .catch(() => {
                        console.error("Failed to fetch accounts");
                    });

                axios
                    .post("/api/@me/transfers", { token: token })
                    .then((response) => {
                        setTransfers(response.data);
                    })
                    .catch(() => {
                        console.error("Failed to fetch transfers");
                    });
                })
                .catch((error) => {
                    console.error("Failed to send money", error.response.data);
                });
    }

    function deleteReceiver (id: number) {
        if (!window.confirm("Are you sure you want to delete this receiver?")) {
            return;
        }
        axios
            .post("/api/@me/remove-receiver", {
                token: token,
                id: id
            })
            .then(() => {
                axios
                    .post("/api/@me/receivers", { token: token })
                    .then((response) => {
                        setReceivers(response.data);
                        setAccountToId(response.data[0].account_id);
                    })
                    .catch(() => {
                        console.error("Failed to fetch receivers");
                    });
            })
            .catch((error) => {
                console.error("Failed to delete receiver", error.response.data);
            });
    }

    function addReceiver (e: React.FormEvent) {
        e.preventDefault();
        axios
            .post("/api/@me/add-receiver", {
                token: token,
                account_id: receiverId,
                name: receiverName,
            })
            .then(() => {
                axios
                    .post("/api/@me/receivers", { token: token })
                    .then((response) => {
                        setReceivers(response.data);
                        setAccountToId(response.data[0].account_id);
                    })
                    .catch(() => {
                        console.error("Failed to fetch receivers");
                    });
            })
            .catch((error) => {
                console.error("Failed to add receiver", error.response.data);
            });
    }

    useEffect(() => {
        const storedToken = localStorage.getItem("token");
        if (storedToken) {
            axios
                .post("/api/@me", { token: storedToken })
                .then((response) => {
                    setUser(response.data);
                })
                .catch(() => {
                    navigate("/login");
                });

            axios
                .post("/api/@me/transfers", { token: storedToken })
                .then((response) => {
                    setTransfers(response.data);
                })
                .catch(() => {
                    console.error("Failed to fetch transfers");
                });

            axios
                .post("/api/@me/accounts", { token: storedToken })
                .then((response) => {
                    setAccounts(response.data);
                    if (response.data.length > 0) {
                        setAccountId(response.data[0].id);
                    }
                })
                .catch(() => {
                    console.error("Failed to fetch accounts");
                });

            axios
                .post("/api/@me/cards", { token: storedToken })
                .then((response) => {
                    setCards(response.data);
                })
                .catch(() => {
                    console.error("Failed to fetch cards");
                });

            axios
                .post("/api/@me/receivers", { token: storedToken })
                .then((response) => {
                    setReceivers(response.data);
                    setAccountToId(response.data[0].account_id);
                })
                .catch(() => {
                    console.error("Failed to fetch receivers");
                });
        } else {
            navigate("/login");
        }
    }, []);

    if (!user) {
        return (
            <div className='bank-page'>
                <TopBar user={user} />
                <div className='bank-section'>
                    <h1>Loading...</h1>
                </div>
            </div>
        );
    }

    return (
        <div className='bank-page'>
            <TopBar user={user} />
            <div className='bank-section'>
                <h1>Home</h1>
                <p>Welcome {user.name} {user.lastname}</p>
            </div>
            <div className='bank-section'>
                <h2>Your Transfers</h2>
                {transfers.length > 0 ? (
                    <ul>
                        {transfers.map((transfer) => (
                            <li key={transfer.id}>
                                Message: {transfer.name}, Amount: {transfer.value}, From: {transfer.sender_name}, To: {transfer.receiver_name}  {transfer.receiver_lastname}
                            </li>
                        ))}
                    </ul>
                ) : (
                    <p>No transfers found.</p>
                )}
            </div>
            {accounts.length > 0 && receivers.length > 0 && (
                <div className='bank-section'>
                    <h2>Send money</h2>
                    <form onSubmit={sendMoney}>
                        <select name="" id="" value={String(accountId)} onChange={(e) => setAccountId(Number(e.target.value))}>
                            {accounts.map((account: any) => (
                                <option key={account.id} value={account.id}>
                                    {account.name}
                                </option>
                            ))}
                        </select>
                        <input type="number" placeholder="Amount" onChange={(e) => setAmount(Number(e.target.value))} value={amount}/>
                        
                        <select name="" id="" value={String(accountToId)} onChange={(e) => setAccountToId(Number(e.target.value))}>
                            {receivers.map((receiver: any) => (
                                <option key={receiver.id} value={receiver.account_id}>
                                    {receiver.name}
                                </option>
                            ))}
                        </select>
                        <input type="text" placeholder="Message" onChange={(e) => setMessage(e.target.value)} value={message}/>
                        <button type="submit">Send</button>
                    </form>
                </div>
            )}
            <div className='bank-section'>
                <h2>Your Accounts</h2>
                {accounts.length > 0 ? (
                    accounts.map((account) => (
                        <div key={account.id}>
                            <p>
                                <strong>Name:</strong> {account.name} - <strong>Balance:</strong> {account.balance} - <strong>Interest:</strong> {account.interest} - <strong>Account ID:</strong> {account.id}
                            </p>
                        </div>
                    ))
                ) : (
                    <p>No accounts found.</p>
                )}
            </div>
            <div className='bank-section'>
                <h2>Your Cards</h2>
                {cards.length > 0 ? (
                    cards.map((card: any) => (
                        <div key={card.id}>
                            <p>
                                <strong>Card Account:</strong> {card.account_name} - <strong>Card Number:</strong> {card.number} - <strong>Expiration:</strong> {card.expiration} - <strong>CVC:</strong> {card.cvc}
                            </p>
                        </div>
                    ))
                ) : (
                    <p>No cards found.</p>
                )}
            </div>
            <div className='bank-section'>
                <h2>Your Receivers</h2>
                {receivers.length > 0 ? (
                    receivers.map((receiver: any) => (
                        <div key={receiver.id}>
                            <p>
                                <strong>Receiver Name:</strong> {receiver.name} - <strong>Account ID:</strong> {receiver.account_id}
                            </p>
                            <button onClick={() => deleteReceiver(receiver.id)}>Delete</button>
                        </div>
                    ))
                ) : (
                    <p>No receivers found.</p>
                )}
            </div>
            <div className='bank-section'>
                <h2>Add Receiver</h2>
                <form onSubmit={addReceiver}>
                    <input type="text" placeholder="Receiver Name" onChange={(e) => setReceiverName(e.target.value)} value={receiverName}/>
                    <input type="number" placeholder="Amount" onChange={(e) => setReceiverId(Number(e.target.value))} value={receiverId}/>
                    <button type="submit">Add</button>
                </form>
            </div>
        </div>
    )
}