import { useState, useEffect } from 'react';
import { useParams, useNavigate } from 'react-router-dom';
import axios from 'axios';
import { User } from '../types';

import TopBar from '../components/TopBar';

export default function UserPage() {
    const navigate = useNavigate();
    const [user, setUser] = useState<User>();
    const [thisUser, setThisUser] = useState<User>();
    const [accounts, setAccounts] = useState([]);
    const [cards, setCards] = useState([]);
    const [name, setName] = useState('');
    const [balance, setBalance] = useState(0);
    const [accountId, setAccountId] = useState<Number>();
    const token = localStorage.getItem('token');
    const { id } = useParams();

    function handleSubmit(event: React.FormEvent) {
        event.preventDefault();

        axios
            .post(`/api/users/${id}/create-account`, { token: token, name: name })
            .then(() => {
                axios
                    .post(`/api/users/${id}/accounts`, { token: token })
                    .then((response) => {
                        setAccounts(response.data);
                        setName('');
                        setAccountId(response.data[0]?.id);
                    })
                    .catch((error) => {
                        console.error('Error fetching user:', error.response.data);
                    });
            })
            .catch((error) => {
                console.error('Error creating account:', error.response.data);
            });
    }

    function deleteAccount(accountId: number) {
        if (!window.confirm('Are you sure you want to delete this account?')) {
            return;
        }
        axios
            .post(`/api/users/${id}/accounts/${accountId}/delete`, { token: token })
            .then(() => {
                axios
                    .post(`/api/users/${id}/accounts`, { token: token })
                    .then((response) => {
                        setAccounts(response.data);
                        setName('');
                        setAccountId(response.data[0]?.id);
                    })
                    .catch((error) => {
                        console.error('Error fetching user:', error.response.data);
                    });
            })
            .catch((error) => {
                console.error('Error deleting account:', error.response.data);
            });
    }

    function addCard(event: React.FormEvent) {
        event.preventDefault();

        axios
            .post(`/api/users/${id}/create-card`, { token: token, account_id: accountId })
            .then(() => {
                axios
                    .post(`/api/users/${id}/cards`, { token: token })
                    .then((response) => {
                        setCards(response.data);
                    })
                    .catch((error) => {
                        console.error('Error fetching user:', error.response.data);
                    });
            })
            .catch((error) => {
                console.error('Error creating account:', error.response.data);
            });
    }

    function deleteCard(cardId: number) {
        if (!window.confirm('Are you sure you want to delete this card?')) {
            return;
        }
        axios
            .post(`/api/users/${id}/cards/${cardId}/delete`, { token: token })
            .then(() => {
                axios
                    .post(`/api/users/${id}/cards`, { token: token })
                    .then((response) => {
                        setCards(response.data);
                    })
                    .catch((error) => {
                        console.error('Error fetching user:', error.response.data);
                    });
            })
            .catch((error) => {
                console.error('Error deleting account:', error.response.data);
            });
    }

    function addBalance(event: React.FormEvent) {
        event.preventDefault();

        axios
            .post(`/api/users/${id}/accounts/${accountId}/add-balance`, { token: token, balance: balance })
            .then(() => {
                axios
                    .post(`/api/users/${id}/accounts`, { token: token })
                    .then((response) => {
                        setAccounts(response.data);
                        setBalance(0);
                        setAccountId(response.data[0]?.id);
                    })
                    .catch((error) => {
                        console.error('Error fetching user:', error.response.data);
                    });
            })
            .catch((error) => {
                console.error('Error creating account:', error.response.data);
            });
    }

    useEffect(() => {
        if (token) {
            axios
                .post('/api/@me', { token: token })
                .then((response) => {
                    setThisUser(response.data);
                })
                .catch((error) => {
                    console.error('Error fetching user:', error.response.data);
                });

            axios
                .post(`/api/users/${id}`, { token: token })
                .then((response) => {
                    setUser(response.data);
                })
                .catch((error) => {
                    console.error('Error fetching user:', error.response.data);
                });

            axios
                .post(`/api/users/${id}/accounts`, { token: token })
                .then((response) => {
                    setAccounts(response.data);
                    setAccountId(response.data[0]?.id);
                })
                .catch((error) => {
                    console.error('Error fetching user:', error.response.data);
                });

            axios
                .post(`/api/users/${id}/cards`, { token: token })
                .then((response) => {
                    setCards(response.data);
                })
                .catch((error) => {
                    console.error('Error fetching cards:', error.response.data);
                });
        } else {
            navigate('/login');
        }
    }, [token]);

    if (!user || !thisUser) {
        return (
            <div className='bank-page'>
                <TopBar user={thisUser} />
                <div className='bank-section'>
                    <h1>Loading...</h1>
                </div>
            </div>
        )
    }

    return (
        <div className='bank-page'>
            <TopBar user={thisUser} />
            <div className='bank-section'>
            <h2>{user.name} {user.lastname}</h2>
                <p><strong>Email:</strong> {user.email}</p>
                <p><strong>Phone Number:</strong> {user.numero}</p>
                <p><strong>Admin:</strong> {user.admin === 1 ? 'Yes' : 'No'}</p>
            </div>
            <div className='bank-section'>
                <h2>User Accounts</h2>
                {accounts.length > 0 ? (
                    accounts.map((account: any) => (
                        <div key={account.id}>
                            <p>
                                <strong>Name:</strong> {account.name} - <strong>Balance:</strong> {account.balance} - <strong>Interest:</strong> {account.interest} - <strong>Account ID:</strong> {account.id}
                            </p>
                            <button onClick={() => deleteAccount(account.id)}>Delete account</button>
                        </div>
                    ))
                ) : (
                    <p>No accounts found.</p>
                )}
            </div>
            {accounts.length > 0 && (
                <div className='bank-section'>
                    <h2>Add balance</h2>
                    <form onSubmit={addBalance}>
                        <select name="" id="" value={String(accountId)} onChange={(e) => setAccountId(Number(e.target.value))}>
                            {accounts.map((account: any) => (
                                <option key={account.id} value={account.id}>
                                    {account.name}
                                </option>
                            ))}
                        </select>
                        <input type="number" placeholder='Balance' id='balance' onChange={(e) => setBalance(Number(e.target.value))} value={balance} />
                        <button type='submit'>Add balance</button>
                    </form>
                </div>
            )}
            <div className='bank-section'>
                <h2>Create Account</h2>
                <form onSubmit={handleSubmit}>
                    <input type="text" placeholder='Account name' id='name' onChange={(e) => setName(e.target.value)} value={name} />
                    <button type="submit">Create account</button>
                </form>
            </div>
            <div className='bank-section'>
                <h2>User Cards</h2>
                {cards.length > 0 ? (
                    cards.map((card: any) => (
                        <div key={card.id}>
                            <p>
                                <strong>Card Account:</strong> {card.account_name} - <strong>Card Number:</strong> {card.number} - <strong>Expiration:</strong> {card.expiration} - <strong>CVC:</strong> {card.cvc}
                            </p>
                            <button onClick={() => deleteCard(card.id)}>Delete card</button>
                        </div>
                    ))
                ) : (
                    <p>No cards found.</p>
                )}
            </div>
            {accounts.length > 0 && (
                <div className='bank-section'>
                    <h2>Add card</h2>
                    <form onSubmit={addCard}>
                        <select name="" id="" value={String(accountId)} onChange={(e) => setAccountId(Number(e.target.value))}>
                            {accounts.map((account: any) => (
                                <option key={account.id} value={account.id}>
                                    {account.name}
                                </option>
                            ))}
                        </select>
                        <button type='submit'>Add card</button>
                    </form>
                </div>
            )}
        </div>
    );
}